@@ -5,7 +5,7 @@ import socket
 from struct import pack
 from hashlib import md5
 from decimal import Decimal
-from collections import deque, defaultdict, OrderedDict
+from collections import deque, defaultdict
 from itertools import count, islice
 from six.moves import map
 from six import (
@@ -492,9 +492,16 @@ def convert_paramstyle(style, query):
                     state = INSIDE_CO
             elif style == "qmark" and c == "?":
                 output_query.append(next(param_idx))
-            elif style == "numeric" and c == ":":
+            elif style == "numeric" and c == ":" and next_c != ':' \
+                    and prev_c != ':':
+                # Treat : as beginning of parameter name if and only
+                # if it's the only : around
+                # Needed to properly process type conversions
+                # i.e. sum(x)::float
                 output_query.append("$")
-            elif style == "named" and c == ":":
+            elif style == "named" and c == ":" and next_c != ':' \
+                    and prev_c != ':':
+                # Same logic for : as in numeric parameters
                 state = INSIDE_PN
                 placeholders.append('')
             elif style == "pyformat" and c == '%' and next_c == "(":
@@ -763,7 +770,7 @@ def int_in(data, offset, length):
     return int(data[offset: offset + length])
 
 
-class Cursor():
+class Cursor(object):
     """A cursor object is returned by the :meth:`~Connection.cursor` method of
     a connection. It has the following attributes and methods:
 
@@ -1055,6 +1062,37 @@ TERMINATE = b('X')
 CLOSE = b('C')
 
 
+def _establish_ssl(_socket, ssl_params):
+    if not isinstance(ssl_params, dict):
+        ssl_params = {}
+
+    try:
+        import ssl as sslmodule
+
+        keyfile = ssl_params.get('keyfile')
+        certfile = ssl_params.get('certfile')
+        ca_certs = ssl_params.get('ca_certs')
+        if ca_certs is None:
+            verify_mode = sslmodule.CERT_NONE
+        else:
+            verify_mode = sslmodule.CERT_REQUIRED
+
+        # Int32(8) - Message length, including self.
+        # Int32(80877103) - The SSL request code.
+        _socket.sendall(ii_pack(8, 80877103))
+        resp = _socket.recv(1)
+        if resp == b('S'):
+            return sslmodule.wrap_socket(
+                _socket, keyfile=keyfile, certfile=certfile,
+                cert_reqs=verify_mode, ca_certs=ca_certs)
+        else:
+            raise InterfaceError("Server refuses SSL")
+    except ImportError:
+        raise InterfaceError(
+            "SSL required but ssl module not available in "
+            "this python installation")
+
+
 def create_message(code, data=b('')):
     return code + i_pack(len(data) + 4) + data
 
@@ -1070,18 +1108,19 @@ STATEMENT = b('S')
 PORTAL = b('P')
 
 # ErrorResponse codes
-RESPONSE_SEVERITY = b("S")  # always present
-RESPONSE_CODE = b("C")  # always present
-RESPONSE_MSG = b("M")  # always present
-RESPONSE_DETAIL = b("D")
-RESPONSE_HINT = b("H")
-RESPONSE_POSITION = b("P")
-RESPONSE__POSITION = b("p")
-RESPONSE__QUERY = b("q")
-RESPONSE_WHERE = b("W")
-RESPONSE_FILE = b("F")
-RESPONSE_LINE = b("L")
-RESPONSE_ROUTINE = b("R")
+RESPONSE_SEVERITY = "S"  # always present
+RESPONSE_SEVERITY = "V"  # always present
+RESPONSE_CODE = "C"  # always present
+RESPONSE_MSG = "M"  # always present
+RESPONSE_DETAIL = "D"
+RESPONSE_HINT = "H"
+RESPONSE_POSITION = "P"
+RESPONSE__POSITION = "p"
+RESPONSE__QUERY = "q"
+RESPONSE_WHERE = "W"
+RESPONSE_FILE = "F"
+RESPONSE_LINE = "L"
+RESPONSE_ROUTINE = "R"
 
 IDLE = b("I")
 IDLE_IN_TRANSACTION = b("T")
@@ -1105,6 +1144,12 @@ class Connection(object):
     NotSupportedError = property(
         lambda self: self._getError(NotSupportedError))
 
+    def __enter__(self):
+        return self
+
+    def __exit__(self, exc_type, exc_value, traceback):
+        self.close()
+
     def _getError(self, error):
         warn(
             "DB-API extension connection.%s used" %
@@ -1113,7 +1158,7 @@ class Connection(object):
 
     def __init__(
             self, user, host, unix_sock, port, database, password, ssl,
-            timeout, application_name, max_prepared_statements):
+            timeout, application_name, max_prepared_statements, tcp_keepalive):
         self._client_encoding = "utf8"
         self._commands_with_count = (
             b("INSERT"), b("DELETE"), b("UPDATE"), b("MOVE"),
@@ -1163,22 +1208,12 @@ class Connection(object):
                 self._usock.connect(unix_sock)
 
             if ssl:
-                try:
-                    import ssl as sslmodule
-                    # Int32(8) - Message length, including self.
-                    # Int32(80877103) - The SSL request code.
-                    self._usock.sendall(ii_pack(8, 80877103))
-                    resp = self._usock.recv(1)
-                    if resp == b('S'):
-                        self._usock = sslmodule.wrap_socket(self._usock)
-                    else:
-                        raise InterfaceError("Server refuses SSL")
-                except ImportError:
-                    raise InterfaceError(
-                        "SSL required but ssl module not available in "
-                        "this python installation")
+                self._usock = _establish_ssl(self._usock, ssl)
 
             self._sock = self._usock.makefile(mode="rwb")
+            if tcp_keepalive:
+                self._usock.setsockopt(
+                    socket.SOL_SOCKET, socket.SO_KEEPALIVE, 1)
         except socket.error as e:
             self._usock.close()
             raise InterfaceError("communication error", e)
@@ -1471,9 +1506,12 @@ class Connection(object):
         self.in_transaction = False
 
     def handle_ERROR_RESPONSE(self, data, ps):
-        msg = OrderedDict(
-            (s[:1], s[1:].decode(self._client_encoding)) for s in
+        msg = dict(
+            (
+                s[:1].decode(self._client_encoding),
+                s[1:].decode(self._client_encoding)) for s in
             data.split(NULL_BYTE) if s != b(''))
+
         response_code = msg[RESPONSE_CODE]
         if response_code == '28000':
             cls = InterfaceError
@@ -1481,7 +1519,8 @@ class Connection(object):
             cls = IntegrityError
         else:
             cls = ProgrammingError
-        self.error = cls(*itervalues(msg))
+
+        self.error = cls(msg)
 
     def handle_EMPTY_QUERY_RESPONSE(self, data, ps):
         self.error = ProgrammingError("query was empty")
@@ -2404,10 +2443,9 @@ def array_has_null(arr):
 
 def array_dim_lengths(arr):
     len_arr = len(arr)
+    retval = [len_arr]
     if len_arr > 0:
         v0 = arr[0]
         if isinstance(v0, list):
-            retval = [len(v0)]
             retval.extend(array_dim_lengths(v0))
-            return retval
-    return [len_arr]
+    return retval
